rem $DBA/fixowner.sql
rem
rem Creates a SQL script to fix the ownership of all primary keys which have
rem a different owner than the table name that they are referencing, and
rem where the primary key is referenced in a foreign key constraint.
rem
rem This causes ORA-00600 errors having an argument of 12838 to appear in
rem trace dump files.
rem
rem Fix this by dropping the foreign key constraints, dropping the primary
rem key, and then recreating the primary key and recreating the foreign key
rem constraints while connecting as the table's owner
rem
rem Example of fix_own.sql script that this script produces:
rem
rem * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
rem
rem fix_own.sql
rem whenever sqlerror exit failure
rem rem ----------------------------
rem rem Fixing bad OPS$ORACLE7 index ownership on table TES_MGR.PORTFOLIO
rem rem
rem alter table TES_MGR.TES_CORP_ACT_ADJ drop constraint TES_CORP_ACT_ADJ_FK1;
rem alter table TES_MGR.PORTFOLIO drop primary key;
rem alter user tes_mgr identified by xyzzy;
rem connect tes_mgr/xyzzy
rem alter table TES_MGR.PORTFOLIO add primary key ("PORTFOLIO_ID") using index
rem  tablespace TES_SCRATCH
rem  storage(initial 32K next 10K pctincrease 50 minextents 1 maxextents 121)
rem  pctfree 10 initrans 2 maxtrans 255;
rem alter table TES_MGR.TES_CORP_ACT_ADJ add constraint TES_CORP_ACT_ADJ_FK1
rem  foreign key ("PORTFOLIO_ID") references TES_MGR.PORTFOLIO ("PORTFOLIO_ID");
rem connect /
rem alter user tes_mgr identified by values '35D30134731C62F0';
rem exit
rem
rem * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
rem
rem Created 10/14/96 by Brian Lomasky
rem
set feedback off
set heading off 
set termout off
set echo off
set pagesize 0
set newpage 0
set linesize 80
set recsep off
set verify off 
set serveroutput on size 100000
define q=chr(39)

rem Cleanup
set termout on
select 'Creating SQL script to fix the badly-owned indexes...' from dual;

rem Create temp table for output report
create table fix_own_temp (
	linen		number,
	lin		varchar2(80));

declare
	t_r_owner		sys.dba_constraints.r_owner%TYPE;
	t_r_table		sys.dba_constraints.table_name%TYPE;
	t_r_constraint_name	sys.dba_constraints.r_constraint_name%TYPE;
	t_delete_rule		sys.dba_constraints.delete_rule%TYPE;
	pk_constraint_name	sys.dba_constraints.constraint_name%TYPE;
	i_owner			sys.dba_indexes.owner%TYPE;
	lineno			number;
	a_lin			varchar2(80);
	user_password		sys.dba_users.password%TYPE;
	connect_pwd		varchar2(10);
	errlin			varchar2(80);

	cursor for_key is
		select	dc1.r_owner, dc2.table_name r_table,
				dc1.r_constraint_name, dc1.delete_rule
		from	dba_constraints dc1, dba_constraints dc2
		where	dc1.r_constraint_name = dc2.constraint_name
		and	dc1.constraint_name not like 'SYS%'
		and	dc1.constraint_type = 'R'
		order by
			1, 2, 3;

	cursor pwd_cursor (c_user varchar2) is
		select password from sys.dba_users where username = c_user;

	function wri(x_lin in varchar2, x_str in varchar2, x_force in number)
		return varchar2 is
	begin
		if length(x_lin) + length(x_str) > 80
		then
			lineno := lineno + 1;
			insert into fix_own_temp values (lineno, x_lin);
			if x_force = 0 then
				return ' '||x_str;
			elsif x_force = 1 then
				lineno := lineno + 1;
				if substr(x_lin,1,2) = '  '
				then
					insert into fix_own_temp values (
						lineno, x_str);
				else
					insert into fix_own_temp values (
						lineno, ' '||x_str);
				end if;
				return '';
			else
				return 'rem '||x_str;
			end if;
		else
			if x_force != 1 then
				return x_lin||x_str;
			else
				lineno := lineno + 1;
				insert into fix_own_temp values (
					lineno, x_lin||x_str);
				return '';
			end if;
		end if;
	end wri;

	-- Procedure to create the SQL script to re-add the foreign keys
	procedure add_foreign_key_cols(pfk_owner in varchar2,
		pfk_constraint_name in varchar2, pfk_table in varchar2,
		fk_table in varchar2, fk_constraint_name in varchar2,
		t_delete_rule in varchar2)
	is
		c_column_name	sys.dba_cons_columns.column_name%TYPE;
		c_position	sys.dba_cons_columns.position%TYPE;

		cursor c2 (pfk_owner in varchar2,
			a_constraint_name in varchar2, pfk_table in varchar2)
		is
			select	column_name, position
			from	dba_cons_columns
			where	owner = pfk_owner
			and	constraint_name = a_constraint_name
			and	table_name = pfk_table
       			order by position;
		begin
			a_lin := wri(a_lin, ' foreign key (', 0);
			open c2 (pfk_owner, pfk_constraint_name, pfk_table);
			loop
				fetch c2 into c_column_name, c_position;
				exit when c2%notfound;
				if c_position != 1 then
					a_lin := wri(a_lin, ', ', 0);
				end if;
				a_lin := wri(a_lin, chr(34) || c_column_name ||
					chr(34), 0);
			end loop;
			close c2;
			a_lin := wri(a_lin, ')', 0);
			a_lin := wri(a_lin, ' references ', 0);
			a_lin := wri(a_lin, pfk_owner||'.'||fk_table, 0);
			a_lin := wri(a_lin, ' (', 0);
			open c2 (pfk_owner, fk_constraint_name, fk_table);
			loop
				fetch c2 into c_column_name, c_position;
				exit when c2%notfound;
				if c_position != 1
				then
					a_lin := wri(a_lin, ', ', 0);
				end if;
				a_lin := wri(a_lin, chr(34) ||
					c_column_name || chr(34), 0);
			end loop;
			close c2;
			a_lin := wri(a_lin, ')', 0);
			if t_delete_rule = 'CASCADE' then
				a_lin := wri(a_lin, ' on delete cascade;', 1);
			else
				a_lin := wri(a_lin, ';', 1);
			end if;
	end add_foreign_key_cols;

	-- Procedure to drop or re-create all of the foreign key constraints
	-- that refer to a given table
	procedure process_foreign_keys(fk_owner in varchar2,
		fk_table in varchar2, drop_create in number,
		t_delete_rule in varchar2)
	is
		pfk_constraint_name sys.dba_constraints.constraint_name%TYPE;
		fk_constraint_name  sys.dba_constraints.r_constraint_name%TYPE;
		pfk_table_name	    sys.dba_constraints.table_name%TYPE;

		cursor pfk_cur (fk_owner in varchar2, fk_table in varchar2) is
			select	c1.constraint_name, c1.table_name,
				c1.r_constraint_name
			from	dba_constraints c1, dba_constraints c2,
				dba_tables t
			where	c1.r_constraint_name = c2.constraint_name
			and	c1.constraint_type = 'R'
			and	c2.table_name = t.table_name
			and	t.owner = fk_owner
			and	t.table_name = fk_table
       			order by 1, 2;
	begin
		-- Locate owner/constraint_name of the referential integrity
		-- constraints which refer to this table
		open pfk_cur (fk_owner, fk_table);
		loop
			fetch pfk_cur into pfk_constraint_name, pfk_table_name,
				fk_constraint_name;
			exit when pfk_cur%notfound;
			a_lin := wri(a_lin, 'alter table ', 0);
			a_lin := wri(a_lin, fk_owner||'.'||pfk_table_name, 0);
			if drop_create = 0 then
				a_lin := wri(a_lin, ' drop constraint ', 0);
				a_lin := wri(a_lin, pfk_constraint_name||';',
					1);
			else
				a_lin := wri(a_lin, ' add constraint ', 0);
				a_lin := wri(a_lin, pfk_constraint_name, 0);
				add_foreign_key_cols(fk_owner,
					pfk_constraint_name, pfk_table_name,
					fk_table, fk_constraint_name,
					t_delete_rule);
			end if;
		end loop;
		close pfk_cur;
	end process_foreign_keys;

	-- Procedure to drop and recreate the primary key for a table under
	-- the table's ownership
	procedure redo_primary_key(pk_own in varchar2,
		pk_cons_name in varchar2, pk_table_name in varchar2)
	is
		c_owner		sys.dba_cons_columns.owner%TYPE;
		c_constraint_name
				sys.dba_cons_columns.constraint_name%TYPE;
		c_table_name	sys.dba_cons_columns.table_name%TYPE;
		c_column_name	sys.dba_cons_columns.column_name%TYPE;
		c_position	sys.dba_cons_columns.position%TYPE;
		tbs_name	sys.dba_indexes.tablespace_name%TYPE;
		ini_tr		sys.dba_indexes.ini_trans%TYPE;
		max_tr		sys.dba_indexes.max_trans%TYPE;
		init_ex		sys.dba_indexes.initial_extent%TYPE;
		next_ex		sys.dba_indexes.next_extent%TYPE;
		min_ex		sys.dba_indexes.min_extents%TYPE;
		max_ex		sys.dba_indexes.max_extents%TYPE;
		pct_inc		sys.dba_indexes.pct_increase%TYPE;
		pct_fr		sys.dba_indexes.pct_free%TYPE;
		initial_extent_size	varchar2(16);
		next_extent_size	varchar2(16);
		missing_pri_index exception;

		cursor c2 (t_owner varchar2, t_constraint_name varchar2,
			t_table_name varchar2) is
			select owner, constraint_name, table_name, column_name,
				position
			from dba_cons_columns
			where owner = t_owner and
				constraint_name = t_constraint_name and
				table_name = t_table_name
         			order by position;
		cursor c5 (t_cons varchar2) is
			select tablespace_name,
				ini_trans, max_trans,
				initial_extent,
				next_extent,
				min_extents,
				max_extents,
				pct_increase, pct_free
				from dba_indexes
				where index_name = t_cons;
		begin
			a_lin := wri(a_lin, 'alter table ', 0);
			a_lin := wri(a_lin, pk_own||'.'||pk_table_name, 0);
			a_lin := wri(a_lin, ' add primary key (', 0);
			open c2 (pk_own, pk_cons_name, pk_table_name);
			loop
				fetch c2 into c_owner,
					c_constraint_name, c_table_name,
					c_column_name, c_position;
				exit when c2%notfound;
				if c_position = 1
				then
					a_lin := wri(a_lin, chr(34) ||
						c_column_name || chr(34), 0);
				else
					a_lin := wri(a_lin, ', ', 0);
					a_lin := wri(a_lin, chr(34) ||
						c_column_name || chr(34), 0);
				end if;
			end loop;
			close c2;
			open c5 (pk_cons_name);
			fetch c5 into tbs_name, ini_tr, max_tr, init_ex,
				next_ex, min_ex, max_ex, pct_inc, pct_fr;
			if c5%notfound then
				raise missing_pri_index;
			end if;
			close c5;
			/* Calculate extent sizes in Mbytes or Kbytes, if */
			/* possible */
			if mod(init_ex, 1048576) = 0
			then
				initial_extent_size :=
					to_char(init_ex / 1048576) || 'M';
			elsif mod(init_ex, 1024) = 0
			then
				initial_extent_size :=
					to_char(init_ex / 1024) || 'K';
			else
				initial_extent_size := to_char(init_ex);
			end if;
			if mod(next_ex, 1048576) = 0
			then
				next_extent_size :=
					to_char(next_ex / 1048576) || 'M';
			elsif mod(next_ex, 1024) = 0
			then
				next_extent_size :=
					to_char(next_ex / 1024) || 'K';
			else
				next_extent_size := to_char(next_ex);
			end if;
			a_lin := wri(a_lin, ') using index ', 0);
			a_lin := wri(a_lin, 'tablespace ', 0);
			a_lin := wri(a_lin, tbs_name, 1);
			a_lin := wri(a_lin, ' storage(', 0);
			a_lin := wri(a_lin, 'initial ', 0);
			a_lin := wri(a_lin, initial_extent_size, 0);
			a_lin := wri(a_lin, ' next ', 0);
			a_lin := wri(a_lin, next_extent_size, 0);
			a_lin := wri(a_lin, ' pctincrease ', 0);
			a_lin := wri(a_lin, pct_inc, 0);
			a_lin := wri(a_lin, ' minextents ', 0);
			a_lin := wri(a_lin, min_ex, 0);
			a_lin := wri(a_lin, ' maxextents ', 0);
			a_lin := wri(a_lin, max_ex, 0);
			a_lin := wri(a_lin, ')', 1);
			a_lin := wri(a_lin, ' pctfree ', 0);
			a_lin := wri(a_lin, pct_fr, 0);
			a_lin := wri(a_lin, ' initrans ', 0);
			a_lin := wri(a_lin, ini_tr, 0);
			a_lin := wri(a_lin, ' maxtrans ', 0);
			a_lin := wri(a_lin, max_tr, 0);
			a_lin := wri(a_lin, ';', 1);
		exception
			when missing_pri_index then
				close c5;
				a_lin := wri(a_lin, ');', 1);
	end redo_primary_key;

begin
	errlin := 'init';
	lineno := 0;
	a_lin := '';
	a_lin := wri(a_lin, 'rem fix_own.sql', 1);
	a_lin := wri(a_lin, 'whenever sqlerror exit failure', 1);
	open for_key;
	loop
		<<fetch_for_key>>
		-- Fetch the referenced table for every foreign key constraint
		fetch for_key into t_r_owner, t_r_table, t_r_constraint_name,
			t_delete_rule;
		exit when for_key%notfound;
		errlin := 'for_key: ' || t_r_table || ', ' ||
			t_r_constraint_name;

		-- Fetch the owner of the primary key on the referenced table
		declare
		cursor pk_const (ref_owner varchar2, ref_table varchar2) is
			select i.owner, c.constraint_name
			from dba_constraints c, dba_tables t, dba_indexes i
			where c.constraint_type = 'P' and
				c.constraint_name = i.index_name and
				c.table_name = i.table_name and
				c.table_name = t.table_name and
				t.owner = ref_owner and
				t.table_name = ref_table;
		begin
			errlin := 'open pk_const';
			open pk_const (t_r_owner, t_r_table);
			-- Fetch the owner of the primary key index on the table
			-- referenced in the foreign key constraint
			fetch pk_const into i_owner, pk_constraint_name;
			if pk_const%notfound then
				dbms_output.put_line('Warning:  Primary' ||
					' key index_name does not match' ||
					' constraint_name for');
				dbms_output.put_line(
					'--->  ' || t_r_owner || '.' ||
					t_r_table);
				close pk_const;
				goto fetch_for_key;
			end if;
			close pk_const;
			errlin := 'pk_const: ' || t_r_owner || ', ' ||
				t_r_table;
			if t_r_owner != i_owner then
				a_lin := wri(a_lin,
					'rem ----------------------------', 1);
				a_lin := wri(a_lin, 'rem Fixing bad ' ||
					i_owner, 2);
				a_lin := wri(a_lin,
					' index ownership on table ', 2);
				a_lin := wri(a_lin,
					t_r_owner||'.'||t_r_table, 2);
				a_lin := wri(a_lin, '', 1);
				a_lin := wri(a_lin, 'rem', 1);
				-- Drop all of the foreign key constraints
				-- that refer to this table
				errlin := 'process_foreign_keys: 0';
				process_foreign_keys(t_r_owner, t_r_table, 0,
					t_delete_rule);
				-- Drop and recreate the primary key for this
				-- table under the table's ownership
				a_lin := wri(a_lin, 'alter table ', 0);
				a_lin := wri(a_lin, t_r_owner||'.'||t_r_table,
					0);
				a_lin := wri(a_lin, ' drop primary key;', 1);
				errlin := 'drop primary';
				/* */
				/* Since we need to create the primary key */
				/* index as the same owner as the table, */
				/* we need to log in as the table's owner */
				/* */
				open pwd_cursor(t_r_owner);
				fetch pwd_cursor into user_password;
				if pwd_cursor%NOTFOUND then
					user_password := '<password>';
					dbms_output.put_line(
						'*****> Warning:' ||
						'  Username '||t_r_owner||
						' not found in DBA_USERS!!');
				end if;
				close pwd_cursor;
				errlin := 'pwd cursor';
				if user_password = '<password>' then
					errlin := 'No pwd';
					connect_pwd := user_password;
					a_lin := wri(a_lin,
						'alter user '||
						lower(t_r_owner)||
						' identified by' ||
						' <password>;', 1);
				else
					errlin := 'xyzzy pwd';
					connect_pwd := 'xyzzy';
					a_lin := wri(a_lin,
						'alter user '||
						lower(t_r_owner)||
						' identified by' ||
						' xyzzy;', 1);
				end if;
				errlin := 'connect';
				a_lin := wri(a_lin, 'connect '||
					lower(t_r_owner)||'/'||connect_pwd, 1);
				errlin := 'redo primary';
				redo_primary_key(t_r_owner, pk_constraint_name,
					t_r_table);
				-- Re-Create all of the foreign key constraints
				-- that refer to this table
				errlin := 'process_foreign_keys: 1';
				process_foreign_keys(t_r_owner, t_r_table, 1,
					t_delete_rule);
				/* Reset password */
				errlin := 'reset pwd';
				a_lin := wri(a_lin, 'connect /', 1);
				if user_password != '<password>' then
					a_lin := wri(a_lin, 'alter user '||
						lower(t_r_owner)||
						' identified by values '||
						&q||user_password||&q||';', 1);
				else
					a_lin := wri(a_lin, 'alter user '||
						lower(t_r_owner)||
						' identified by <password>;',
						1);
				end if;
			end if;
		end;
	end loop;
	a_lin := wri(a_lin, 'exit', 1);
	close for_key;
	commit;
exception
	when others then
		rollback;
		raise_application_error(-20000,
			'Unexpected error on ' || t_r_table ||
			'.' || t_r_constraint_name || ': ' ||
			to_char(SQLCODE) || ', After: ' || errlin ||
			' - Aborting...');
end;
/
set termout off
spool fix_own.sql
select rtrim(lin) from fix_own_temp order by linen;
spool off
set termout on
prompt Created fix_own.sql
drop table fix_own_temp;
exit
