REM   --------------------------------------------------------------------------
REM   Qsetup.sql   
REM
REM   Copyright(c) 1996,1997,1998 by Savant Corporation
REM   All rights reserved.             
REM   ---------------------------------------------------------------------------
REM
REM   This script is normally run from the Q installer program.
REM    
REM   To execute manually, connect as the SYS user to target database and issue 
REM   following command:
REM    
REM   "start <install_dir>\qsql\qsetup.sql <install_dir> <connect_string> <Q tablespace> 
REM         <temporary_tablespace>"
REM
REM      (e.g "start C:\SAVANT\qsql\qsetup.sql  C:\SAVANT dev1 QDATA TEMP")
REM   
REM   Command-line parameters:
REM
REM   <install_dir>  is a path to the Q installation directory, must be writeable.
REM   <connect string> is a database connect string for target database.
REM   <Q tablespace> is a tablespace name for the QDBA objects.
REM   <temporary_tablespace> is temp tablespace name for QDBA user.
REM           
REM   The script will check that this database is at least v7.2 of ORACLE7
REM   and also that the necessary Oracle supplied packages are installed and
REM   valid.  Any failure of the checks will abort the installation.
REM
REM   If all checks are OK, then the installation will continue.  First the QDBA
REM   user will be created and granted DBA role and numerous direct privileges on
REM   ORACLE7 catalog objects.  Then the Q system objects will be created.
REM 
REM   Script execution is logged to the file <install_dir>\qsql\qsetup.log.  This file
REM   should be checked for any errors.  See qoracle.txt for instructions on how to 
REM   handle common errors.
REM   -----------------------------------------------------------------------------
/* ==================================================== */
/*            Revision History                          */ 
/* ==================================================== */
--  01/29/98: added q110_11a.ddl upgrade from 1.0 [JMB]
--            and beta
--  01/27/98: executing q110_11a.ddl to upgrade  [JMB]
--            to 1.1a
--  11/24/97: check for partial (false beta)     [JMB]
--            install with table exists
--  11/14/97: added call to QDBAGRNT.DCL         [JMB]
--  11/14/97: added grant SELECT_CATALOG_ROLE    [JMB]
--  11/14/97: added create of Q_USER_ROLE        [JMB]
--  10/30/97: bug: needed END in anon block      [JMB]
--  10/29/97: modified to call separately wrapped [JMB]
--            files for package specs/bodies
--  10/29/97: added qora.wps and qora.wpb        [JMB]
--  10/27/97: added check on Q$INSTANCE view     [JMB]
--  10/27/97: added Oracle8 as valid version     [JMB]
--  09/30/97: dropped DBA role and moved all     [JMB]
--            priv grants into qsysvws.ddl
--  06/20/97: dropped Q$LCKUSR from checking     [JMB]
--  06/15/97: added set timing off               [JMB]
--  06/15/97: upgrade to v1.1.0 from v1.0.0      [JMB]
--  00/00/96: original creation                  [JMB]
/* ==================================================== */

undefine install_dir
undefine connect_string
undefine qtablespace
undefine temptablespace

DEFINE install_dir=&1
DEFINE connect_string=&2
DEFINE qtablespace=&3
DEFINE temptablespace=&4

set serveroutput on size 1000000
set timing off
set arraysize 1
set verify off
set feedback off
set define ON

clear screen

var check_return_code NUMBER

set termout on
prompt
prompt ***********************************************
prompt *  Qsetup:  environment pre-check
prompt ***********************************************
prompt

REM ----------------------------------
REM     get ORACLE7 version
REM ----------------------------------

set termout off
col O7 new_value O7ver
SELECT TRANSLATE(SUBSTR(version,1,3),'~.','~') O7
  FROM product_component_version
 WHERE UPPER(product) LIKE 'ORACLE7%'
    OR UPPER(product) LIKE 'PERSONAL ORACLE7%'
    OR UPPER(product) LIKE 'ORACLE8%';
set termout on

REM -------------------------------------------
REM force abort if unsupported ORACLE7 version
REM or user not connected as SYS
REM -------------------------------------------

BEGIN
   :check_return_code := NVL(:check_return_code,0);
   IF USER != 'SYS'
   THEN
      :check_return_code := :check_return_code +1;
      dbms_output.put_line('ERROR: not connected as SYS user');
   ELSE
      dbms_output.put_line('OK, connected as SYS');
   END IF;
   IF '&&O7ver' NOT IN ('72','73','80')
   THEN
      :check_return_code := :check_return_code +1;
      dbms_output.put_line('ERROR: ORACLE version must be 7.2, 7.3 or 8.0');
   ELSE
      dbms_output.put_line('OK, ORACLE version is &&O7ver');
   END IF;
   IF :check_return_code > 0
   THEN
      dbms_output.put_line('Pre-check failed, installation will exit.');
   ELSE
      dbms_output.put_line('Pre-check OK, installation will continue.');
   END IF;
END;
/

rem >>> pause for 5 secs to display message
BEGIN
   dbms_lock.sleep(5);
END;
/

spool &&install_dir\qsql\qchkall.log
start &&install_dir\qsql\qchkall.sql &&install_dir\qsql
spool off

WHENEVER SQLERROR EXIT
BEGIN
   IF :check_return_code > 0
   THEN
      raise_application_error(-20000, 'exiting...');
   END IF;
END;
/
WHENEVER SQLERROR CONTINUE

prompt
prompt ******************************************************
prompt *  Qsetup:  installing QDBA user and database objects
prompt ******************************************************
prompt

set feedback off

undefine qdba
undefine qdbapassword

DEFINE qdba='QDBA'
DEFINE qdbapassword='QDBA'

REM figure out connectstring
col cs new_value connectstring
set termout off

select decode('&&connect_string','','',
           '@'||TRANSLATE('&&connect_string','~@','~') ) cs
from dual;

spool &&install_dir\qsql\qtemp.sql

REM ----------------------------------
REM create QDBA user and GRANT privs
REM ----------------------------------
prompt prompt
prompt prompt Preparing user &&qdba...
prompt prompt
DECLARE
   FUNCTION role_exists(role_IN IN VARCHAR2) RETURN BOOLEAN
   IS
      temp_int INTEGER :=0;
   BEGIN
      select count(1) 
        into temp_int 
        from dba_roles 
       where role=UPPER(role_IN);
      --
      RETURN temp_int > 0;
   END role_exists;
   --
BEGIN
   -- create QDBA if not there
   IF q$instl.Qver = q$instl.q_not_installed
   THEN
      dbms_output.put_line('start &&install_dir\qsql\qdba.dcl');
   END IF;
   -- create role if not there
   IF NOT role_exists('Q_USER_ROLE')
   THEN
      dbms_output.put_line('CREATE ROLE Q_USER_ROLE;');
   END IF;
   -- grant new O8 roles if there
   IF role_exists('SELECT_CATALOG_ROLE') AND role_exists('EXECUTE_CATALOG_ROLE')
   THEN
      dbms_output.put_line('GRANT SELECT_CATALOG_ROLE TO &&qdba;');
      dbms_output.put_line('GRANT EXECUTE_CATALOG_ROLE TO &&qdba;');
      dbms_output.put_line('GRANT SELECT_CATALOG_ROLE TO Q_USER_ROLE;');
      dbms_output.put_line('GRANT EXECUTE_CATALOG_ROLE TO Q_USER_ROLE;');
   END IF;
END;
/

REM moved all grants into qsysvws.ddl and removed DBA role from qdba
REM
REM prompt GRANT CREATE SESSION, DBA TO &&qdba;;
prompt ALTER USER &&qdba DEFAULT TABLESPACE &&qtablespace QUOTA UNLIMITED ON &&qtablespace;;
prompt ALTER USER &&qdba TEMPORARY TABLESPACE &&temptablespace;;

prompt prompt
prompt prompt Creating SYS views and GRANTS to &&qdba user...
prompt prompt
prompt set feedback off
prompt start &&install_dir\qsql\qsysvws.ddl
prompt GRANT ALL ON q$instl TO qdba;;
prompt set feedback on

prompt prompt 
prompt prompt The installation script will now connect to &&connect_string
prompt prompt as QDBA and create the Q system objects, including VIEWS, TABLES
prompt prompt and PACKAGES.  This may take awhile.
prompt prompt
prompt prompt

prompt BEGIN
prompt   dbms_lock.sleep(5);;
prompt END;;
prompt /

prompt set termout off
prompt col oldpw new_value old_password
prompt select password oldpw
prompt  from dba_users
prompt where username = 'QDBA';;

prompt ALTER USER qdba IDENTIFIED BY &&qdbapassword;;
prompt set termout on

prompt prompt
prompt prompt Connecting as &&qdba to database &&connectstring...
prompt prompt

prompt connect &&qdba/&&qdbapassword&&connectstring
set define OFF 
prompt set feedback off
prompt ALTER USER qdba IDENTIFIED BY VALUES '&&old_password';;
prompt set feedback on
prompt set serveroutput on size 1000000
set define ON
prompt prompt 
prompt prompt Creating Q tables and views...
prompt prompt

DECLARE
   FUNCTION table_exists(table_IN IN VARCHAR2) RETURN BOOLEAN
   IS
      temp_int INTEGER :=0;
   BEGIN
      select count(1) 
        into temp_int 
        from user_tables 
       where table_name = UPPER(table_IN);
      --
      RETURN temp_int > 0;
   END table_exists;
   --
BEGIN
   IF sys.q$instl.Qver = sys.q$instl.q_not_installed
   THEN
      dbms_output.put_line('start &&install_dir\qsql\qtables.ddl');
      dbms_output.put_line('start &&install_dir\qsql\qdetails.ddl');
   ELSIF
      sys.q$instl.Qver = sys.q$instl.q_beta_version
   THEN
      -- check for partial or incomplete installs and run
      -- everything if true
      --
      IF    NOT table_exists('Q$INSTAT_LOG') 
         OR NOT table_exists('Q$BGP_DEBUG')
         OR NOT table_exists('Q$SEG')
      THEN
         dbms_output.put_line('start &&install_dir\qsql\qtables.ddl');
         dbms_output.put_line('start &&install_dir\qsql\qdetails.ddl');
      ELSE 
      -- 
      -- OK really was a beta upgrade, not failed install
      --
         dbms_output.put_line('start &&install_dir\qsql\qbeta100.ddl');
         dbms_output.put_line('start &&install_dir\qsql\qdetails.ddl');
         dbms_output.put_line('start &&install_dir\qsql\q100_110.ddl');
         dbms_output.put_line('start &&install_dir\qsql\q110_11a.ddl');
      END IF;
   ELSIF
      sys.q$instl.Qver = '1.0.0'
   THEN
      dbms_output.put_line('start &&install_dir\qsql\q100_110.ddl');
      dbms_output.put_line('start &&install_dir\qsql\q110_11a.ddl');
   ELSIF
      sys.q$instl.Qver = '1.1.0'
   THEN
      dbms_output.put_line('start &&install_dir\qsql\q110_11a.ddl');
   END IF;
END;
/
REM *******************************************
REM prompt start &&install_dir\qsql\qviews.ddl
REM *******************************************
prompt start &&install_dir\qsql\qviews&&O7ver..ddl

REM pause ...

prompt prompt 
prompt prompt Creating Q package specifications...
prompt prompt

BEGIN
   IF sys.q$instl.O7ver IN ('72','73')
   THEN
      dbms_output.put_line('start &&install_dir\qsql\qdecl.wps');
      dbms_output.put_line('start &&install_dir\qsql\qcvar.wps');
      dbms_output.put_line('start &&install_dir\qsql\qinstat.wps');
      dbms_output.put_line('start &&install_dir\qsql\qbgproc.wps'); 
      dbms_output.put_line('start &&install_dir\qsql\qsess.wps');
      dbms_output.put_line('start &&install_dir\qsql\qinst2.wps');
      dbms_output.put_line('start &&install_dir\qsql\qtrend.wps');
      dbms_output.put_line('start &&install_dir\qsql\qspace.wps');
      dbms_output.put_line('start &&install_dir\qsql\qrepl.wps');
      dbms_output.put_line('start &&install_dir\qsql\qora.wps');
      dbms_output.put_line('start &&install_dir\qsql\qdetl&&O7ver..wps');

   ELSIF sys.q$instl.O7ver IN ('80','81')
   THEN
      dbms_output.put_line('start &&install_dir\qsql\qdecl80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qcvar80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qinst80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qbgp80.wps'); 
      dbms_output.put_line('start &&install_dir\qsql\qsess80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qinst280.wps');
      dbms_output.put_line('start &&install_dir\qsql\qtrend80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qspace80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qrepl80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qora80.wps');
      dbms_output.put_line('start &&install_dir\qsql\qdetl80.wps');
   END IF;
END;
/

prompt prompt
prompt prompt Creating/compiling Q package bodies.  This process 
prompt prompt can take up to 20 minutes for all the Q packages. 
prompt prompt  
prompt prompt Please be patient...
prompt prompt

rem pause to display message
prompt BEGIN
prompt    dbms_lock.sleep(3);;
prompt END;;
prompt /

REM   Flush shared pool a couple of times to make sure (hopefully) that
REM   enough room exists for Q package compilations.

prompt set feedback off
prompt ALTER SYSTEM FLUSH SHARED_POOL;;
prompt /
prompt set feedback on

BEGIN
   IF sys.q$instl.O7ver IN ('72','73')
   THEN
      dbms_output.put_line('start &&install_dir\qsql\qdecl.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qcvar.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qinstat.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qbgproc.wpb'); 
      dbms_output.put_line('start &&install_dir\qsql\qsess.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qinst2.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qtrend.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qspace.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qrepl.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qora.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qdetl&&O7ver..wpb');

   ELSIF sys.q$instl.O7ver IN ('80','81')
   THEN
      dbms_output.put_line('start &&install_dir\qsql\qdecl80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qcvar80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qinst80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qbgp80.wpb'); 
      dbms_output.put_line('start &&install_dir\qsql\qsess80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qinst280.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qtrend80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qspace80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qrepl80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qora80.wpb');
      dbms_output.put_line('start &&install_dir\qsql\qdetl80.wpb');
   END IF;
END;
/

REM  NEW: grants from QDBA to Q_USER_ROLE
prompt start &&install_dir\qsql\qdbagrnt.dcl

prompt set termout off
prompt select * from user_errors where type like 'PACK%' and name like 'Q%';;
prompt set termout on

spool off

set termout on
spool &&install_dir\qsql\qsetup.log
start &&install_dir\qsql\qtemp.sql

set termout on
prompt
prompt ***********************************************
prompt *  Qsetup  verify installation
prompt ***********************************************
prompt
set feedback on
BEGIN
   :check_return_code :=0;
   --
   sys.q$instl.check_object('SYS','Q$LATCH','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('SYS','Q$BH','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('SYS','Q$SPFREE','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('SYS','Q$KSPPI','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('SYS','Q$LOCK','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('SYS','Q$LOCKC','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   --
   sys.q$instl.check_object('&&qdba','Q$CIRCUIT','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$OBJCHSUM','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$TBSSEG','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$FILFREE','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$AVG_IO','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$LCKCON','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$SESSTAT','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$SESSTOT','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$SESSIO','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$TBSFREE','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$TBSBLKS','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$INSTANCE','VIEW'
         ,:check_return_code,sys.q$instl.info_msg_level);
--
-- q$lckusr view obsoleted in v1.1
--   sys.q$instl.check_object('&&qdba','Q$LCKUSR','VIEW'
--         ,:check_return_code,sys.q$instl.info_msg_level);
   --
   sys.q$instl.check_object('&&qdba','Q$INSTAT','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$INSTAT','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$DECL','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$DECL','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$INST2','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$INST2','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$SESS','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$SESS','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$BGPROC','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$BGPROC','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$TREND','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$TREND','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$CVAR','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$CVAR','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$SPACE','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$SPACE','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$DETL','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$DETL','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$REPL','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$REPL','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$ORA','PACKAGE'
         ,:check_return_code,sys.q$instl.info_msg_level);
   sys.q$instl.check_object('&&qdba','Q$ORA','PACKAGE BODY'
         ,:check_return_code,sys.q$instl.info_msg_level);
   --
END;
/

GRANT EXECUTE ON q$bgproc to SYS,SYSTEM;
DROP PACKAGE SYS.Q$INSTL;

prompt 
prompt Results of installation verification:
prompt

BEGIN
   --
   IF :check_return_code = 0 
   THEN
      dbms_output.put_line('All Q objects OK, starting background process');
      q$bgproc.start_driver;
      dbms_output.put_line('SUCCESS');
   ELSE
      dbms_output.put_line
         ('Detected '||:check_return_code||' ERRORS with Q objects.');
      dbms_output.put_line
         ('See &&install_dir\qsql\qsetup.log for details.');
   END IF;
END;
/

spool off

var temp_count NUMBER
BEGIN
   select count(*) into :temp_count from q$instat_log;
END;
/

rem pause to display message
BEGIN
   dbms_lock.sleep(5);
   IF :check_return_code = 0 AND :temp_count < 2
   THEN
      dbms_output.put_line('Loading startup data for Q Viewer...2 minutes');
   END IF;
END;
/

DECLARE
   temp_seq	INTEGER;
   temp_Instance_rec    q$instat.Instance_rectype;
   temp_return_code     q$decl.return_code_var%TYPE;
   temp_return_key      q$decl.return_key_var%TYPE;
BEGIN
   --
   -- only initialize if this is initial install based on no data in table
   IF :check_return_code = 0 AND :temp_count < 2
   THEN 
   --
      DELETE FROM q$instat_log;
      --
	   FOR i IN 1..4 LOOP
         --
		   SELECT q$instat_seq.nextval INTO temp_seq FROM dual;
         --
         q$instat.get_Instance (temp_instance_rec, temp_return_code, temp_return_key);
         --
		   INSERT INTO q$instat_log
		   VALUES   (temp_seq
			         ,sysdate-50/1440+10*i/1440
			         ,50+100*MOD(i,2)	/* collect_hsecs */
			         ,q$decl.instance_start_date
			         ,temp_Instance_rec.InstID.log_sequence
                  ,temp_Instance_rec.Sysstats.logons
                  ,temp_Instance_rec.Sysstats.logons_cum
                  ,temp_Instance_rec.Sysstats.open_cursors
                  ,temp_Instance_rec.Sysstats.open_cursors_cum
                  ,temp_Instance_rec.Sysstats.commits
			         ,(i*100)		       /* db_block_gets */
			         ,0
			         ,(i-1)*54+13*MOD(i,2)	/* physical_reads */
                  ,temp_Instance_rec.SGAactiv.buffer_busy_waits
                  ,temp_Instance_rec.SGAactiv.buffer_busy_waittime
                  ,temp_Instance_rec.Sysstats.enqueue_requests
                  ,temp_Instance_rec.Sysstats.enqueue_timeouts
                  ,temp_Instance_rec.Sysstats.redo_size
                  ,temp_Instance_rec.Sysstats.user_calls
                  ,temp_Instance_rec.Sysstats.parse_count
                  ,temp_Instance_rec.Sysstats.execute_count
                  ,temp_Instance_rec.Sysstats.recursive_calls
                  ,temp_Instance_rec.Sysstats.sorts_disk
                  ,temp_Instance_rec.Sysstats.sorts_memory
                  ,temp_Instance_rec.Sysstats.table_scan_rows_gotten
                  ,temp_Instance_rec.Sysstats.table_fetch_rowid
                  ,temp_Instance_rec.Sysstats.table_scans_short
                  ,temp_Instance_rec.Sysstats.table_scans_long
                  ,temp_Instance_rec.Sysstats.table_fetch_cont_row
                  ,temp_Instance_rec.SGAactiv.rbs_waits
                  ,temp_Instance_rec.SGAactiv.rbs_gets
                  ,temp_Instance_rec.SGAactiv.rbs_xacts
                  ,(i*100)                /* latch_gets */
                  ,(i-1)*54+13*MOD(i,2)      /* latch_misses */
                  ,temp_Instance_rec.SGAactiv.latch_sleeps
                  ,temp_Instance_rec.SGAactiv.latch_imm_gets
                  ,temp_Instance_rec.IOstats.dbfile_waits
                  ,temp_Instance_rec.IOstats.dbfile_waittime
                  ,temp_Instance_rec.IOstats.lgfile_waits
                  ,temp_Instance_rec.IOstats.lgfile_waittime
                  ,(i*100)               /* dicache_gets */
                  ,(i-1)*54+13*MOD(i,2)     /* dicache_getmisses */
                  ,(i*100)               /* libcache_gets   */
                  ,(i-1)*54+13*MOD(i,2)    /* libcache_gethits */
                  ,temp_Instance_rec.SGAactiv.libcache_pins
                  ,temp_Instance_rec.SGAactiv.libcache_pinhits
                  ,temp_Instance_rec.SGAactiv.libcache_reloads
                  ,temp_Instance_rec.SGAdata.shpool_free_KB
                  ,temp_Instance_rec.SGAdata.shpool_maxfree_KB
                  ,temp_Instance_rec.SGAdata.shpool_free_exts
                  ,0
                  ,temp_Instance_rec.IOstats.dbf_physrds
                  ,temp_Instance_rec.IOstats.dbf_physwrts
                  ,temp_Instance_rec.SGAactiv.active_tempsegs
                  ,temp_Instance_rec.MTSdata.disp_busy_hsec
                  ,temp_Instance_rec.MTSdata.disp_idle_hsec
                  ,temp_Instance_rec.MTSdata.disp_waittime_hsec
                  ,temp_Instance_rec.MTSdata.disp_totalqueue
                  ,temp_Instance_rec.MTSdata.serv_busy_hsec
                  ,temp_Instance_rec.MTSdata.serv_idle_hsec
                  ,temp_Instance_rec.MTSdata.serv_waittime_hsec
                  ,temp_Instance_rec.MTSdata.serv_totalqueue
			         ,TO_CHAR(sysdate-50/1440+10*i/1440,'HH24')
			         ,TO_CHAR(sysdate-50/1440+10*i/1440,'DY')
                 );
         dbms_lock.sleep(15);
	   END LOOP;
      commit work;
   END IF;
END;
/

exit
